var DataSync = (function (api) {
    
    var myModule = {};
    var dsDevice;
    var advanced = false;
    var saveDisabled = true;
    let SERVICE_ID = "urn:constantsphere-com:serviceId:DataSync";

    function getDevices() {
        Devices = [];
        try {
            Variables = JSON.parse(api.getDeviceState(dsDevice, SERVICE_ID, "loggedIds"));
        } catch(err) { 
            console.log("Error decoding loggedIds");
            Variables = []; 
        }
        try {
            Services = JSON.parse(api.getDeviceState(dsDevice, SERVICE_ID, "serviceList"));
        } catch (err) { 
            console.log("Error decoding serviceList");
            Services = []; 
        }
        console.log("loggedIds"+Variables.length)
        var devices = api.getUserData().devices;
        for (let device of devices) {
            for (let state of device.states) {
                let inService = Services.find(x=> {return x.Service == state.service && x.Variable == state.variable});
                    let inVariable = Variables.find(x=> {return x.D == device.id && x.S == inService?.Id && x.E == 1});
                    let thisDevice = {
                        "Id":(inVariable?.Id==null)?-1:inVariable?.Id,
                        "Enabled":(inVariable?.E==null)?1:inVariable?.E,
                        "Device":device.id,
                        "Service":state.service,
                        "Variable":state.variable,
                        "Name":device.name,
                        "Log":(inVariable!=null),
                        "ServiceId":inService?.Id
                    }
                    Devices.push(thisDevice);
            }
        }
    }

    function setAdvanced(a) {
        advanced = a;
        var html = buildDeviceList(advanced);
        api.setCpanelContent(html);
    }

    function showConstantGraph() {
        dsDevice = api.getCpanelDeviceId();
        var allDevices = api.getListOfDevices()
        var dmDevice = allDevices.find(x => {return x.device_type=="urn:schemas-cd-jackson-com:device:DataMine:1"});
        var dmState = null;
        if (dmDevice) {dmState = dmDevice.states.find(x => {return x.variable == "SetDataDirectory"});}
        var enabled = (api.getDeviceState(dsDevice, SERVICE_ID, "enabled") == "1");
        var apiKey = api.getDeviceState(dsDevice, SERVICE_ID, "apiKey") || "";
        var status = api.getDeviceState(dsDevice, SERVICE_ID, "Status") || "initialising";
        var version = api.getDeviceState(dsDevice, SERVICE_ID, "version") || "";
        var noCertCheck =  (api.getDeviceState(dsDevice, SERVICE_ID, "noCertCheck")=="1");
        var syncWithDataMine = (api.getDeviceState(dsDevice, SERVICE_ID, "syncWithDataMine") == "1");
        var dataMineDir = api.getDeviceState(dsDevice, SERVICE_ID, "dataMineDirectory") || "";
        var html = "<p>To log data to ConstantGraph get your API Key from <a href='https://www.constantgraph.com'>ConstantGraph</a></p>";
        html+= `<p>API Key: <input type="text" id="dsApiKey" class="ds-input" value='${apiKey}' onInput="document.getElementById('dsSaveApi').disabled = false"> <input type="button" id="dsSaveApi" class="btn btn-success" value="Save" disabled onClick="DataSync.saveAPIKey()" /></p>
                <p>Last status: <b>${status}</b></p>
                <p>Logging enabled: 
                    <div class="on-off-device-container deviceButtonContainer width_75">
                    <div data-container="button" class="on-off-device cursor_pointer ${(enabled)?"on":""}" onClick="DataSync.setVariable(this,'enabled')"></div>
                    <div data-container="label" class="stateLabel">${(enabled)?"ON":"OFF"}</div></div></p>
                <p>Disable certificate check (may help with connection difficulties on older boxes)&nbsp;
                <input type="checkbox" ${(noCertCheck)?"checked='checked'":""} onClick="DataSync.setSimpleVar('noCertCheck',(this.checked)?1:0)" /></p>
                <p>Sync with dataMine 2.0: (requires product subscription)<br/>
                    <i>it looks like dataMine is ${(dmDevice==null)?"not installed":"installed and the data directory is "+dmState?.value}</i>
                    <div class="on-off-device-container deviceButtonContainer width_75">
                    <div data-container="button" class="on-off-device cursor_pointer ${(syncWithDataMine)?"on":""}" onClick="DataSync.setVariable(this,'syncWithDataMine')"></div>
                    <div data-container="label" class="stateLabel">${(syncWithDataMine)?"ON":"OFF"}</div></div></p>
                <p>dataMine 2.0 directory: <input type="text" id="dmDirectory" class="ds-input" value='${dataMineDir}' onInput="document.getElementById('dsSaveDir').disabled = false"> <input type="button" id="dsSaveDir" class="btn btn-success" value="Save" disabled onClick="DataSync.saveDMDir()" /></p>
                <p>&copy; ConstantGraph version <a href="/port_3480/data_request?id=lr_dsDebug_${dsDevice}">${version}</a>.`;

        api.setCpanelContent(html);
    }

    function setSimpleVar(variable,value) {
        api.setDeviceStatePersistent(dsDevice, SERVICE_ID, variable, value);
    }

    function setVariable(object,variable) {
        value = !object.className.endsWith("on");
        api.setDeviceStatePersistent(dsDevice, SERVICE_ID, variable, (value)?1:0);
        console.log(`set ${variable} to: ${value}`);
        object.className = `on-off-device cursor_pointer ${(value)?"on":""}`;
        object.nextElementSibling.innerText=`${(value)?"ON":"OFF"}`;
    }

    function saveDMDir() {
        var dataMineDir = document.getElementById("dmDirectory").value;
        console.log(`save DM Dir: ${dataMineDir}`);
        api.setDeviceStatePersistent(dsDevice, SERVICE_ID, "dataMineDirectory", dataMineDir);
        document.getElementById("dsSaveDir").disabled = true;
    }

    function saveAPIKey() {
        var apiKey = document.getElementById("dsApiKey").value;
        api.setDeviceStatePersistent(dsDevice, SERVICE_ID, "apiKey", apiKey);
        document.getElementById("dsSaveApi").disabled = true;
    }
     
    function showDevices() {
        dsDevice = api.getCpanelDeviceId();
        var syncWithDataMine = api.getDeviceState(dsDevice, SERVICE_ID, "syncWithDataMine");
        console.log(`dsDevice:${dsDevice} syncWithDM:${syncWithDataMine}`);
        var html="";

        if(syncWithDataMine==1) {
            html = `<h3>Logged devices set to be controlled by dataMine 2</h3>
                <p>Click to open <a href="/dm/">dataMine 2</a> to edit logged devices</p>`;
            api.setCpanelContent(html);
            return
        }

        try {
            getDevices();
            html = buildDeviceList(advanced);
        } catch (err) {
            html = `<h3>Waiting for server to start</h3>
                <p>Try again in a minute...</p>`;
        }
        api.setCpanelContent(html);
    }

    function buildDeviceList(advanced) {
        var style = '';
        let thisDevice = 0;

        var html=`<div class="btn-group btn-group-toggle" >
                <label class="btn btn-secondary ${(advanced)?"":"active"}"><input type="radio" autocomplete="off" ${(advanced)?"":"checked"} onClick="DataSync.setAdvanced(false);"> Basic </label>
                <label class="btn btn-secondary ${(advanced)?"active":""}"><input type="radio" autocomplete="off" ${(advanced)?"checked":""} onClick="DataSync.setAdvanced(true);"> Advanced </label>
                </div>`;
        html+= `<table style="width:90%;">
                <tr style="background: #fcfcfc;border: solid #e9ecef;border-width: 1px 0 1px;height:3em"><th>Device</th><th>Description</th><th>Log</th></tr>`

        Devices.forEach( row=> {
            if(advanced && row.Device!=thisDevice) {
                thisDevice = row.Device;
                html+= `<tr style="background: #fcfcfc;border: solid #e9ecef;border-width: 1px 0 1px;height:2.5em"><th>${row.Device}</th><th>${row.Name}</th><th></th>`;
            }
            if(advanced || row.ServiceId!=undefined) {
               if(advanced) {
                    html+= `<tr style="${style}"><td></td><td>${row.Service} - ${row.Variable}</td>`;
                } else {
                    html+= `<tr style="${style}"><td>${row.Device}</td><td>${row.Name} - ${row.Variable}</td>`;
                }
                html+= `<td><input type="checkbox" ${(row.Log)?"checked='checked'":""} onClick="DataSync.selectForLogging(${row.Device},'${row.Service}','${row.Variable}',this.checked)" /></td></tr>`;
                style=(style=="")?"background: #fcfcfc;border: solid #e9ecef;border-width: 1px 0 1px":"";
            }
        });
        html += `</table><br/>
            <p><input type="button" class="btn btn-success" id="dsSave" value="Save" ${(saveDisabled)?"disabled":""} onClick="DataSync.saveVariables()"/></p><br/>`;
        return html;
    }

    function selectForLogging(deviceId,s,v,checked) {
        saveDisabled=false;
        document.getElementById("dsSave").disabled=saveDisabled;
        var device = Devices.find(x=> {return x.Device == deviceId && x.Service == s && x.Variable == v});
        var variable = Variables.find(x=> {return x.D == deviceId && x.S == device.ServiceId});
        device.Log = checked;
        console.log(`set device ${deviceId}:${device.ServiceId} to Log=${checked}`);
        if(variable==undefined && checked) {
            if(device.ServiceId==undefined){
                device.ServiceId = Services.length+1;
                Services.push({"Id":device.ServiceId,"Service":s,"Variable":v});
            }
            Variables.push({"I":-1,"D":deviceId,"S":device.ServiceId,"E":1});
            console.log(`added additional variable D:${deviceId} SId:${device.ServiceId}`);
        } else if(variable!=undefined) {
            variable.E = (checked)?1:0;
        }
    }

    function saveVariables() {
        api.setDeviceStatePersistent(dsDevice, SERVICE_ID, "loggedIds", JSON.stringify(Variables));
        api.setDeviceStatePersistent(dsDevice, SERVICE_ID, "serviceList", JSON.stringify(Services));
        console.log("save Variables");
        saveDisabled=true;
        document.getElementById("dsSave").disabled=saveDisabled;
    }
    
    let Devices = [];
    let Variables = [];  
    let Services = [];

    myModule = {
        uuid: 'e9814081-507f-41e5-8696-125b2a98f429',
        showDevices: showDevices,
        selectForLogging: selectForLogging,
        saveVariables: saveVariables,
        setAdvanced: setAdvanced,
        showConstantGraph: showConstantGraph,
        saveAPIKey: saveAPIKey,
        saveDMDir: saveDMDir,
        setVariable: setVariable,
        setSimpleVar: setSimpleVar
    };
    return myModule;
})(api);